@@ -1,32 +1,16 @@
-;;; htz.el --- Timezone-based time and date support for Hyperbole.
-
-;; Copyright (C) 1991-1995 Free Software Foundation, Inc.
-;; Developed with support from Motorola Inc.
+;;; htz.el ---  Timezone-based time and date support for GNU Hyperbole
+;;
+;; Author:       Masanobu Umeda             / Bob Weiner
+;;
+;; Orig-Date:    14-Oct-91 at 07:22:08
+;;
+;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; See the "HY-COPY" file for license information.
+;;
+;; Derived from "timezone.el" in GNU Emacs.
 ;;
-;; Adapted from Timezone package for GNU Emacs
-;; Copyright(C) 1990 Masanobu UMEDA (umerin@mse.kyutech.ac.jp)
-
-;; Author: Bob Weiner, Brown U.
-;; Maintainer: Mats Lidell <matsl@contactor.se>
-;; Keywords: calendar, hypermedia
-
 ;; This file is part of GNU Hyperbole.
-
-;; GNU Hyperbole is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 3, or (at
-;; your option) any later version.
-
-;; GNU Hyperbole is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
+;;
 ;;; Commentary:
 ;;
 ;; All date parsing functions accept the output of any other parsing
@@ -36,21 +20,20 @@
 ;; turn of the century.
 ;;
 ;; Hyperbole uses this package to normalize all worldwide date times to
-;; Greenwich Mean Time, so that Hyperbole buttons created by users in
-;; different timezones will sort by time properly.
-;;
+;; Universal Time (Greenwich Mean Time), so that Hyperbole buttons
+;; created by users in different timezones will sort by time properly.
 
 ;;; Code:
-
-;;;
+;;; ************************************************************************
 ;;; Other required Elisp libraries
-;;;
+;;; ************************************************************************
 
-(require 'hypb)
+(require 'calendar)
+(require 'cal-julian)
 
-;;;
+;;; ************************************************************************
 ;;; Public functions
-;;;
+;;; ************************************************************************
 
 (defun htz:date-arpa (&optional date local timezone)
   "Convert optional DATE or current date to an arpanet standard date.
@@ -58,71 +41,80 @@ Optional 1st argument LOCAL specifies the default local timezone of the DATE.
 Optional 2nd argument TIMEZONE specifies a timezone to be represented in."
   (or (vectorp date)
       (setq date (htz:date-parse (or date (current-time-string)))))
-  (let* ((year   (string-to-int (aref date 0)))
-	 (month  (string-to-int (aref date 1)))
-	 (day    (string-to-int (aref date 2)))
-	 (time   (htz:time-parse (aref date 3)))
-	 (hour   (string-to-int (aref time 0)))
-	 (minute (string-to-int (aref time 1)))
-	 (second (string-to-int (aref time 2)))
+  (let* ((year   (string-to-number (aref date 0)))
+	 (month  (string-to-number (aref date 1)))
+	 (day    (string-to-number (aref date 2)))
+	 (time   (htz:time-parse   (aref date 3)))
+	 (hour   (string-to-number (aref time 0)))
+	 (minute (string-to-number (aref time 1)))
+	 (second (string-to-number (aref time 2)))
 	 (local  (or (aref date 4) local htz:local)) ;Use original if defined
 	 (timezone (or timezone local))
 	 (diff   (- (htz:zone-to-hour timezone)
 		    (htz:zone-to-hour local)))
 	 (new    (htz:time-fix year month day
-				    (+ hour diff) minute second)))
+			       (+ hour diff) minute second)))
     (htz:date-make-arpa (aref new 0) (aref new 1) (aref new 2)
-			     (htz:time-make-string
-			      (aref new 3) (aref new 4) (aref new 5))
-			     timezone)))
+			(htz:time-make-string
+			 (aref new 3) (aref new 4) (aref new 5))
+			timezone)))
 
 (defun htz:date-parse (date &optional parsed-current-date)
   "Parse DATE string and return a vector [year month day time timezone].
 19 is prepended to year if necessary.  Timezone in DATE is optional, it
-defaults to the value of htz:local.
+defaults to the value of `htz:local'.
 
 Recognizes the following styles:
- (1) 14 Apr 89 03:20[:12] [GMT]
- (2) Fri, 17 Mar [19]89 4:01[:33] [GMT]
- (3) Mon Jan 16 16:12[:37] [GMT] 1989
- (4) 19911014:07:51:08 or 1991101407:51:08  'sortable date'
- (5) Mar 29 14:00    'ls -l date'  requires 'parsed-current-date' arg
- (6) Mar  7  1994    'ls -l date'  requires 'parsed-current-date' arg"
+ (1) \"(1 30 1999)\" or \"(1 30 1999)\"  `calendar-julian-date'  requires `parsed-current-date' arg
+ (2) \"14 Apr 89 03:20[:12] [GMT]\"
+ (3) \"Fri, 17 Mar [19]89 4:01[:33] [GMT]\"
+ (4) \"Mon Jan 16 16:12[:37] [GMT] 1989\"
+ (5) \"19911014:07:51:08 or 1991101407:51:08\"  `sortable date'
+ (6) \"Mar 29 14:00\"    `ls -l date'  requires `parsed-current-date' arg
+ (7) \"Mar  7  1994\"    `ls -l date'  requires `parsed-current-date' arg"
   (let ((date (or date ""))
 	(year nil)
 	(month nil)
 	(day nil)
 	(time nil)
-	(zone nil)) ; This may be nil.
-    (cond ((string-match
-"\\([0-9][0-9][0-9][0-9]\\)\\([0-1][0-9]\\)\\([0-3][0-9]\\):?\\([0-2][0-9]:[0-5][0-9:]+\\)[ ]*\\'" date)
-	   ;; Style (4)
-	   (setq year 1 month 2 day 3 time 4 zone nil))
-	  ((string-match
-"\\([0-9]+\\) \\([^ ,]+\\) \\([0-9]+\\) \\([0-9]+:[0-9:]+\\)[ ]*\\'" date)
-	   ;; Styles: (1) and (2) without timezone
-	   (setq year 3 month 2 day 1 time 4 zone nil))
-	  ((string-match
-"\\([0-9]+\\) \\([^ ,]+\\) \\([0-9]+\\) \\([0-9]+:[0-9:]+\\)[ ]*\\([-+a-zA-Z0-9]+\\)" date)
-	   ;; Styles: (1) and (2) with timezone and buggy timezone
-	   (setq year 3 month 2 day 1 time 4 zone 5))
-	  ((string-match
-"\\([^ ,]+\\) +\\([0-9]+\\) \\([0-9]+:[0-9:]+\\) \\([0-9]+\\)" date)
-	   ;; Styles: (3) without timezone
-	   (setq year 4 month 1 day 2 time 3 zone nil))
-	  ((string-match
-"\\([^ ,]+\\) +\\([0-9]+\\) \\([0-9]+:[0-9:]+\\) \\([-+a-zA-Z0-9]+\\) \\([0-9]+\\)" date)
-	   ;; Styles: (3) with timezone
-	   (setq year 5 month 1 day 2 time 3 zone 4))
-	  ((string-match "^\\([^ ,]+\\) +\\([0-9]+\\) +\\([0-9]+:[0-9:]+\\)$" date)
-	   ;; Style: (5)
-	   (setq year nil month 1 day 2 time 3 zone nil))
-	  ((string-match
-	    "^\\([^ ,]+\\) +\\([0-9]+\\) +\\([0-9][0-9][0-9][0-9]\\)$" date)
-	   ;; Style: (6)
-	   (setq year 3 month 1 day 2 time nil zone nil))
-	  (t (error "(htz:date-parse): Invalid date format: '%s'" date)))
-    (if year
+	(zone nil))			; This may be nil.
+    (if (listp date)
+	(setq month (nth 0 date)
+	      day   (nth 1 date)
+	      year  (nth 2 date))
+      (cond ((string-match
+	      "\\`(\\([0-9]+\\) \\([0-9]+\\) \\([0-9]+\\))\\'" date)
+	     ;; Style (1)
+	     (setq year 3 month 1 day 2 time nil zone nil))
+	    ((string-match
+	      "\\([0-9][0-9][0-9][0-9]\\)\\([0-1][0-9]\\)\\([0-3][0-9]\\):?\\([0-2][0-9]:[0-5][0-9:]+\\)[ ]*\\'" date)
+	     ;; Style (4)
+	     (setq year 1 month 2 day 3 time 4 zone nil))
+	    ((string-match
+	      "\\([0-9]+\\) \\([^ ,]+\\) \\([0-9]+\\) \\([0-9]+:[0-9:]+\\)[ ]*\\'" date)
+	     ;; Styles: (1) and (2) without timezone
+	     (setq year 3 month 2 day 1 time 4 zone nil))
+	    ((string-match
+	      "\\([0-9]+\\) \\([^ ,]+\\) \\([0-9]+\\) \\([0-9]+:[0-9:]+\\)[ ]*\\([-+a-zA-Z0-9]+\\)" date)
+	     ;; Styles: (1) and (2) with timezone and buggy timezone
+	     (setq year 3 month 2 day 1 time 4 zone 5))
+	    ((string-match
+	      "\\([^ ,]+\\) +\\([0-9]+\\) \\([0-9]+:[0-9:]+\\) \\([0-9]+\\)" date)
+	     ;; Styles: (3) without timezone
+	     (setq year 4 month 1 day 2 time 3 zone nil))
+	    ((string-match
+	      "\\([^ ,]+\\) +\\([0-9]+\\) \\([0-9]+:[0-9:]+\\) \\([-+a-zA-Z0-9]+\\) \\([0-9]+\\)" date)
+	     ;; Styles: (3) with timezone
+	     (setq year 5 month 1 day 2 time 3 zone 4))
+	    ((string-match "^\\([^ ,]+\\) +\\([0-9]+\\) +\\([0-9]+:[0-9:]+\\)$" date)
+	     ;; Style: (5)
+	     (setq year nil month 1 day 2 time 3 zone nil))
+	    ((string-match
+	      "^\\([^ ,]+\\) +\\([0-9]+\\) +\\([0-9][0-9][0-9][0-9]\\)$" date)
+	     ;; Style: (6)
+	     (setq year 3 month 1 day 2 time nil zone nil))
+	    (t (error "(htz:date-parse): Invalid date format: `%s'" date)))
+      (if year
 	  (setq year
 		(substring date (match-beginning year) (match-end year))
 		year (if (/= (length year) 2) year
@@ -131,26 +123,26 @@ Recognizes the following styles:
 			      (century (substring yr 0 2)))
 			 (concat (if (string< curr-yr yr)
 				     (format "%02d"
-					     (1- (string-to-int century)))
+					     (1- (string-to-number century)))
 				   century)
 				 year))))
-      (setq year (if (vectorp parsed-current-date)
-		     (aref parsed-current-date 0)
-		   "0")))
-    (if month
-	(setq month (substring date
-			       (match-beginning month) (match-end month))
-	      month (if (/= (string-to-int month) 0) month
-		      (int-to-string
-		       (cdr (assoc (upcase month) htz:months-assoc)))))
-      (setq month (if (vectorp parsed-current-date)
-		      (aref parsed-current-date 1)
-		    "0")))
-    (if day
-	(setq day (substring date (match-beginning day) (match-end day)))
-      (setq day (if (vectorp parsed-current-date)
-		    (aref parsed-current-date 2)
-		  "0")))
+	(setq year (if (vectorp parsed-current-date)
+		       (aref parsed-current-date 0)
+		     "0")))
+      (if month
+	  (setq month (substring date
+				 (match-beginning month) (match-end month))
+		month (if (/= (string-to-number month) 0) month
+			(int-to-string
+			 (cdr (assoc (upcase month) htz:months-assoc)))))
+	(setq month (if (vectorp parsed-current-date)
+			(aref parsed-current-date 1)
+		      "0")))
+      (if day
+	  (setq day (substring date (match-beginning day) (match-end day)))
+	(setq day (if (vectorp parsed-current-date)
+		      (aref parsed-current-date 2)
+		    "0"))))
     (if time
 	(setq time (substring date (match-beginning time) (match-end time)))
       (setq time (if (vectorp parsed-current-date)
@@ -169,22 +161,22 @@ Optional 1st argument LOCAL specifies the local timezone of the DATE.
 Optional 2nd argument TIMEZONE specifies an output timezone to use."
   (or (vectorp date)
       (setq date (htz:date-parse (or date (current-time-string)))))
-  (let* ((year   (string-to-int (aref date 0)))
-	 (month  (string-to-int (aref date 1)))
-	 (day    (string-to-int (aref date 2)))
-	 (time   (htz:time-parse (aref date 3)))
-	 (hour   (string-to-int (aref time 0)))
-	 (minute (string-to-int (aref time 1)))
-	 (second (string-to-int (aref time 2)))
+  (let* ((year   (string-to-number (aref date 0)))
+	 (month  (string-to-number (aref date 1)))
+	 (day    (string-to-number (aref date 2)))
+	 (time   (htz:time-parse   (aref date 3)))
+	 (hour   (string-to-number (aref time 0)))
+	 (minute (string-to-number (aref time 1)))
+	 (second (string-to-number (aref time 2)))
 	 (local  (or (aref date 4) local htz:local)) ;Use original if defined
 	 (timezone (or timezone local))
 	 (diff   (- (htz:zone-to-hour timezone)
 		    (htz:zone-to-hour local)))
 	 (new    (htz:time-fix year month day
-				    (+ hour diff) minute second)))
+			       (+ hour diff) minute second)))
     (htz:date-make-sortable (aref new 0) (aref new 1) (aref new 2)
-				 (htz:time-make-string
-				  (aref new 3) (aref new 4) (aref new 5)))))
+			    (htz:time-make-string
+			     (aref new 3) (aref new 4) (aref new 5)))))
 
 ;;;
 ;;; Parsers and Constructors of Date and Time
@@ -199,24 +191,23 @@ Optional argument LOCAL specifies the local timezone of the DATE."
   "Convert DATE or current date to a unix standard date.
 Optional 1st argument LOCAL specifies the local timezone of the DATE (default
 is the timezone embedded in the date or if there is none, then the value of
-the variable, htz:local).  Optional 2nd argument TIMEZONE specifies the
-timezone in which the date is returned; it defaults to the value of
-htz:local."
+`htz:local').  Optional 2nd argument TIMEZONE specifies the timezone in which
+the date is returned; it defaults to the value of `htz:local'."
   (or (vectorp date)
       (setq date (htz:date-parse (or date (current-time-string)))))
   (or local (setq local (or (aref date 4) htz:local)))
-  (let* ((year   (string-to-int (aref date 0)))
-	 (month  (string-to-int (aref date 1)))
-	 (day    (string-to-int (aref date 2)))
+  (let* ((year   (string-to-number (aref date 0)))
+	 (month  (string-to-number (aref date 1)))
+	 (day    (string-to-number (aref date 2)))
 	 (time   (htz:time-parse (aref date 3)))
-	 (hour   (string-to-int (aref time 0)))
-	 (minute (string-to-int (aref time 1)))
-	 (second (string-to-int (aref time 2)))
+	 (hour   (string-to-number (aref time 0)))
+	 (minute (string-to-number (aref time 1)))
+	 (second (string-to-number (aref time 2)))
 	 (timezone (or timezone local))
 	 (diff   (- (htz:zone-to-hour timezone)
 		    (htz:zone-to-hour local)))
 	 (fixed    (htz:time-fix year month day
-				    (+ hour diff) minute second)))
+				 (+ hour diff) minute second)))
     (htz:date-make-unix
      (aref fixed 0) (aref fixed 1) (aref fixed 2)
      (htz:time-make-string (aref fixed 3) (aref fixed 4) (aref fixed 5))
@@ -224,22 +215,24 @@ htz:local."
 
 (defun htz:span-in-days (start-date end-date)
   "Return span in days between START-DATE and END-DATE strings.
-See 'htz:date-parse' for a list of acceptable date formats."
-  (require 'calendar)
-  (let* ((parsed-current-date (htz:date-parse (current-time-string)))
-	 (htz-start-date (htz:date-parse start-date parsed-current-date))
-	 (htz-end-date   (htz:date-parse end-date parsed-current-date))
-	 (cal-start-date
-	  (list (string-to-int (aref htz-start-date 1))   ;; month
-		(string-to-int (aref htz-start-date 2))   ;; day
-		(string-to-int (aref htz-start-date 0)))) ;; year
-	 (cal-end-date
-	  (list (string-to-int (aref htz-end-date 1))     ;; month
-		(string-to-int (aref htz-end-date 2))     ;; day
-		(string-to-int (aref htz-end-date 0))))   ;; year
-	 )
-    (- (calendar-absolute-from-julian cal-end-date)
-       (calendar-absolute-from-julian cal-start-date))))
+See `htz:date-parse' for a list of acceptable date formats."
+  (if (and (listp start-date) (listp end-date))
+      (- (calendar-julian-to-absolute end-date)
+	 (calendar-julian-to-absolute start-date))
+    (let* ((parsed-current-date (htz:date-parse (current-time-string)))
+	   (htz-start-date (htz:date-parse start-date parsed-current-date))
+	   (htz-end-date   (htz:date-parse end-date parsed-current-date))
+	   (cal-start-date
+	    (list (string-to-number (aref htz-start-date 1));; month
+		  (string-to-number (aref htz-start-date 2));; day
+		  (string-to-number (aref htz-start-date 0))));; year
+	   (cal-end-date
+	    (list (string-to-number (aref htz-end-date 1));; month
+		  (string-to-number (aref htz-end-date 2));; day
+		  (string-to-number (aref htz-end-date 0))));; year
+	   )
+      (- (calendar-julian-to-absolute cal-end-date)
+	 (calendar-julian-to-absolute cal-start-date)))))
 
 (defun htz:time-parse (time)
   "Parse TIME (HH:MM:SS) and return a vector [hour minute second]."
@@ -262,9 +255,9 @@ See 'htz:date-parse' for a list of acceptable date formats."
      (if second
 	 (substring time (match-beginning second) (match-end second)) "0"))))
 
-;;;
+;;; ************************************************************************
 ;;; Private functions
-;;;
+;;; ************************************************************************
 
 (defun htz:date-make-arpa (year month day time &optional timezone)
   "Make arpanet standard date string from YEAR, MONTH, DAY, and TIME.
@@ -340,26 +333,35 @@ Optional argument TIMEZONE specifies a time zone."
 			      htz:world-timezones))
 		  (and (fboundp 'current-time-zone)
 		       (if (listp (current-time-zone))
-			   (car (cdr (current-time-zone)))
+			   (car (current-time-zone))
 			 (current-time-zone)))
 		  timezone))
 	(if (stringp timezone)
-	    (setq timezone (string-to-int timezone)))
+	    (setq timezone (string-to-number timezone)))
 	(/ timezone 100))
     (error "(htz:zone-to-hour): Nil timezone sent as argument")))
 
 
-;;;
+;;; ************************************************************************
 ;;; Private variables
-;;;
+;;; ************************************************************************
 
 (defvar htz:local
-  (or (getenv "TZ") (getenv "TIMEZONE")
-      (cadr (current-time-zone))
-      (hypb:call-process-p
-       "date" nil '(if (re-search-backward
-			" \\([-+a-zA-Z0-9]+\\) [0-9]+$" nil t)
-		       (buffer-substring (match-beginning 1) (match-end 1)))))
+  (let ((local-tz (or (getenv "TZ") (getenv "TIMEZONE")
+		      (if (fboundp 'current-time-zone)
+			  (car (cdr (current-time-zone))))
+		      (progn 
+			(require 'hypb)
+			(hypb:call-process-p
+			 "date" nil '(if (re-search-backward
+					  " \\([-+a-zA-Z0-9]+\\) [0-9]+$" nil t)
+					 (buffer-substring (match-beginning 1)
+							   (match-end 1))))))))
+    (if (and (stringp local-tz) (string-match " " local-tz))
+	;; Windows returns things like "Eastern Daylight Time", so
+	;; abbreviate to the first letter of each word.
+	(concat (mapcar (lambda (s) (aref s 0)) (split-string local-tz)))
+      local-tz))
   "Holds string giving the timezone for the local machine.")
 
 (defvar htz:world-timezones
@@ -371,8 +373,9 @@ Optional argument TIMEZONE specifies a time zone."
     ("CDT" .  -500)
     ("EST" .  -500)
     ("EDT" .  -400)
-    ("AST" .  -400)			;by <clamen@CS.CMU.EDU>
-    ("NST" .  -330)			;by <clamen@CS.CMU.EDU>
+    ("AST" .  -400)
+    ("NST" .  -330)
+    ("UT"  .  +000)
     ("GMT" .  +000)
     ("BST" .  +100)
     ("MET" .  +100)
@@ -386,10 +389,8 @@ Optional argument TIMEZONE specifies a time zone."
     ("GMT-4"  .  -400) ("GMT-5"  .  -500) ("GMT-6"  .  -600)
     ("GMT-7"  .  -700) ("GMT-8"  .  -800) ("GMT-9"  .  -900)
     ("GMT-10" . -1000) ("GMT-11" . -1100) ("GMT-12" . -1200))
-  "*Time differentials of timezone from GMT in +-HHMM form.
-This list is obsolescent, and is present only for backwards compatibility,
-because time zone names are ambiguous in practice.
-Use `current-time-zone' instead.")
+  "Time differentials of timezone from GMT in +-HHMM form.
+Use `current-time-zone' instead where possible.")
 
 (defvar htz:months-assoc
   '(("JAN" .  1)("FEB" .  2)("MAR" .  3)
