@@ -1,50 +1,43 @@
-;;; kfile.el --- Save and restore kotls from files.
-
-;; Copyright (C) 1995, 2004, 2006 Free Software Foundation, Inc.
-;; Developed with support from Motorola Inc.
-
-;; Author: Bob Weiner, Brown U.
-;;         Kellie Clark
-;; Maintainer: Mats Lidell <matsl@contactor.se>
-;; Keywords: outlines, wp
-
+;;; kfile.el --- Save and restore koutlines from files
+;;
+;; Author:       Bob Weiner
+;;
+;; Orig-Date:    10/31/93
+;;
+;; Copyright (C) 1993-2016  Free Software Foundation, Inc.
+;; See the "../HY-COPY" file for license information.
+;;
 ;; This file is part of GNU Hyperbole.
 
-;; GNU Hyperbole is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 3, or (at
-;; your option) any later version.
-
-;; GNU Hyperbole is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
 ;;; Commentary:
 
 ;;; Code:
-
-;;;
+;;; ************************************************************************
 ;;; Other required Elisp libraries
-;;;
+;;; ************************************************************************
 
-(mapcar 'require '(kproperty kotl-mode))
+(eval-and-compile (mapc #'require '(kproperty kmenu kview)))
 
-;;;
+;;; ************************************************************************
 ;;; Public variables
-;;;
+;;; ************************************************************************
 
 (defconst kfile:version "Kotl-4.0"
   "Version number of persistent data format used for saving koutlines.")
 
-;;;
+;;; Ensure that outline structure data is hidden from view after a file save.
+(add-hook 'after-save-hook #'kfile:narrow-to-kcells)
+
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defvar kfile:escape-newlines t 
+  "Value of print-escape-newlines used by `kfile:print-to-string' function.")
+
+;;; ************************************************************************
 ;;; Entry Points
-;;;
+;;; ************************************************************************
 
 ;;;###autoload
 (defun kfile:find (file-name)
@@ -64,12 +57,26 @@ Return the new kview."
     ;; Finding the file may have already done a kfile:read as invoked through
     ;; kotl-mode via a file local variable setting.  If so, don't read it
     ;; again.
-    (if (kview:is-p kview)
-	nil
+    (unless (kview:is-p kview)
       (kfile:read buffer existing-file))
     (or (eq major-mode 'kotl-mode) (kotl-mode))
     kview))
 
+;;;###autoload
+(defun kfile:is-p ()
+  "Iff current buffer contains an unformatted or formatted koutline, return file format version string, else nil."
+  (let (ver-string)
+    (save-excursion
+      (save-restriction
+	(widen)
+	(goto-char (point-min))
+	(condition-case ()
+	    (progn
+	      (setq ver-string (read (current-buffer)))
+	      (and (stringp ver-string) (string-match "^Kotl-" ver-string)
+		   ver-string))
+	  (error nil))))))
+
 ;;;###autoload
 (defun kfile:view (file-name)
   "View an existing kotl version-2 file FILE-NAME in a read-only mode."
@@ -79,17 +86,16 @@ Return the new kview."
   (let ((existing-file (file-exists-p file-name)))
     (if existing-file
 	(if (not (file-readable-p file-name))
-	    (error
-	     "(kfile:view): \"%s\" is not readable.  Check permissions."
-	     file-name))
-      (error "(kfile:view): \"%s\" does not exist."))
+	    (error "(kfile:view): \"%s\" is not readable.  Check permissions."
+		   file-name))
+      (error "(kfile:view): \"%s\" does not exist." file-name))
     (view-file file-name))
     (kfile:narrow-to-kcells)
     (goto-char (point-min)))
 
-;;;
+;;; ************************************************************************
 ;;; Public functions
-;;;
+;;; ************************************************************************
 
 (defun kfile:create (buffer)
   "Create a new koutline file attached to BUFFER, with a single empty level 1 kotl cell.
@@ -142,48 +148,35 @@ Return file's kview."
       (kimport:file import-from (current-buffer))
       ;; If import buffer name starts with a space, kill it, as it is no
       ;; longer needed.
-      (if (= ?\ (aref (buffer-name import-from) 0))
+      (if (eq ?\ (aref (buffer-name import-from) 0))
 	  (kill-buffer import-from)))
 
     view))
 
-;;;###autoload
-(defun kfile:is-p ()
-  "Iff current buffer contains an unformatted or formatted koutline, return file format version string, else nil."
-  (let (ver-string)
-    (save-excursion
-      (save-restriction
-	(widen)
-	(goto-char (point-min))
-	(condition-case ()
-	    (progn
-	      (setq ver-string (read (current-buffer)))
-	      (and (stringp ver-string) (string-match "^Kotl-" ver-string)
-		   ver-string))
-	  (error nil))))))
-
-(defun kfile:read (buffer existing-file-p)
+(defun kfile:read (buffer existing-file-p &optional ver-string)
   "Create a new kotl view by reading BUFFER or create an empty view when EXISTING-FILE-P is nil.
+Optional VER-STRING is the outline format version number for the BUFFER that
+was previously read by calling `kfile:is-p'.
+
 Return the new view."
-  (let (ver-string)
-    (cond ((not (bufferp buffer))
-	   (error "(kfile:read): Argument must be a buffer, '%s'." buffer))
-	  ((not existing-file-p)
-	   (kfile:create buffer))
-	  ((progn
-	     (set-buffer buffer)
-	     (not (setq ver-string (kfile:is-p))))
-	   (error "(kfile:read): '%s' is not a koutline file." buffer))
-	  ((equal ver-string "Kotl-4.0")
-	   (kfile:read-v4-or-v3 buffer nil))
-	  ((equal ver-string "Kotl-3.0")
-	   (kfile:read-v4-or-v3 buffer t))
-	  ((equal ver-string "Kotl-2.0")
-	   (kfile:read-v2 buffer))
-	  ((equal ver-string "Kotl-1.0")
-	   (error "(kfile:read): V1 koutlines are no longer supported"))
-	  (t (error "(kfile:read): '%s' has unknown kotl version, %s."
-		    buffer ver-string)))))
+  (cond ((not (bufferp buffer))
+	 (error "(kfile:read): Argument must be a buffer, `%s'." buffer))
+	((and (zerop (buffer-size)) (not existing-file-p))
+	 (kfile:create buffer))
+	((progn
+	   (set-buffer buffer)
+	   (not (or (stringp ver-string) (setq ver-string (kfile:is-p)))))
+	 (error "(kfile:read): `%s' is not a koutline file." buffer))
+	((equal ver-string "Kotl-4.0")
+	 (kfile:read-v4-or-v3 buffer nil))
+	((equal ver-string "Kotl-3.0")
+	 (kfile:read-v4-or-v3 buffer t))
+	((equal ver-string "Kotl-2.0")
+	 (kfile:read-v2 buffer))
+	((equal ver-string "Kotl-1.0")
+	 (error "(kfile:read): V1 koutlines are no longer supported"))
+	(t (error "(kfile:read): `%s' has unknown kotl version, %s."
+		  buffer ver-string))))
 
 (defun kfile:read-v2 (buffer)
   "Create a kotl view by reading kotl version-2 BUFFER.  Return the new view."
@@ -232,8 +225,7 @@ If V3-FLAG is true, read as a version-3 buffer."
     ;; Skip past cell contents here.
     (search-forward "\n\^_" nil t 2)
     ;; Read rest of file data.
-    (if v3-flag
-	nil ;; V3 files did not store viewspecs.
+    (unless v3-flag ;; V3 files did not store viewspecs.
       (kvspec:initialize)
       (setq kvspec:current (read)))
     (setq cell-count (read)
@@ -277,7 +269,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
 	   (error "(kfile:update): File \"%s\" is not writable." file)))
     (let* ((buffer-read-only)
 	   (id-counter (kcell:get-attr top 'id-counter))
-	   (kotl-data (make-vector (1+ id-counter) nil))
+	   (kcell-data (make-vector (1+ id-counter) nil))
 	   (standard-output (current-buffer))
 	   (opoint (set-marker (make-marker) (point)))
 	   (kcell-num 1)
@@ -286,18 +278,17 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
       ;; Prepare cell data for saving.
       (kfile:narrow-to-kcells)
       (kview:map-tree
-	(function
-	  (lambda (view)
-	    (setq cell (kcell-view:cell))
-	    (aset kotl-data
-		  kcell-num
-		  (kotl-data:create cell))
-	    (setq kcell-num (1+ kcell-num))))
+        (lambda (view)
+	  (setq cell (kcell-view:cell))
+	  (aset kcell-data
+		kcell-num
+		(kcell-data:create cell))
+	  (setq kcell-num (1+ kcell-num)))
 	kview t)
       ;; Save top cell, 0, last since above loop may increment the total
       ;; number of cells counter stored in it, if any invalid cells are
       ;; encountered. 
-      (aset kotl-data 0 (kotl-data:create top))
+      (aset kcell-data 0 (kcell-data:create top))
       (setq id-counter (kcell:get-attr top 'id-counter))
       ;;
       (widen)
@@ -324,7 +315,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
 		     kvspec:current id-counter label-type label-min-width
 		     label-separator level-indent))
       (princ "\^_\n;; depth-first kcell attributes\n")
-      (kfile:pretty-print kotl-data)
+      (kfile:pretty-print kcell-data)
       ;;
       ;; Don't re-narrow buffer by default since this is used in
       ;; write-contents-hooks after save-buffer has widened buffer.  If
@@ -338,8 +329,7 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
       (set-marker opoint nil)
       nil)))
 
-;;; Next function is adapted from 'file-write' of GNU Emacs 19, copyright FSF,
-;;; under the GPL.
+;;; Next function is adapted from `file-write' of GNU Emacs.
 (defun kfile:write (file)
   "Write current outline to FILE."
   (interactive "FWrite outline file: ")
@@ -355,14 +345,14 @@ VISIBLE-ONLY-P is non-nil.  Signal an error if kotl is not attached to a file."
   ;; koutlines properly, so reinitialize local variables.
   (kotl-mode)
   (set-buffer-modified-p t)
-  ;; This next line must come before the save-buffer since write-file-hooks
+  ;; This next line must come before the save-buffer so write-file-functions
   ;; can make use of it.
-  (kview:set-buffer-name kview (buffer-name))
+  (kview:set-buffer kview (current-buffer))
   (save-buffer))
 
-;;;
+;;; ************************************************************************
 ;;; Private functions
-;;;
+;;; ************************************************************************
 
 (defun kfile:build-structure-v2 (kotl-structure cell-data)
   "Build cell list from the KOTL-STRUCTURE and its CELL-DATA.
@@ -372,28 +362,25 @@ from a koutline file.
 Return list of outline cells in depth first order.  Invisible top cell is not
 included in the list."
   (let ((stack) (sibling-p) (cell-list) func cell)
-    (mapcar
-     (function
-      (lambda (item)
-	(setq func (cdr (assoc item
-			       (list
-				(cons "\("
-				      (function
-				       (lambda ()
-					 (setq stack (cons sibling-p stack)
-					       sibling-p nil))))
-				(cons "\)" 
-				      (function
-				       (lambda ()
-					 (setq sibling-p (car stack)
-					       stack (cdr stack)))))))))
-	(cond (func (funcall func))
-	      ;; 0th cell was created with kview:create.
-	      ((equal item 0) nil)
-	      (t (setq cell (kotl-data:to-kcell-v2 (aref cell-data item))
-		       cell-list (cons cell cell-list)
-		       sibling-p t)
-		 ))))
+    (mapc
+     (lambda (item)
+       (setq func (cdr (assoc item
+			      (list
+			       (cons "\("
+				     (lambda ()
+				       (setq stack (cons sibling-p stack)
+					     sibling-p nil)))
+			       (cons "\)" 
+				     (lambda ()
+				       (setq sibling-p (car stack)
+					     stack (cdr stack))))))))
+       (cond (func (funcall func))
+	     ;; 0th cell was created with kview:create.
+	     ((equal item 0) nil)
+	     (t (setq cell (kcell-data:to-kcell-v2 (aref cell-data item))
+		      cell-list (cons cell cell-list)
+		      sibling-p t)
+		)))
      kotl-structure)
     (nreverse cell-list)))
 
@@ -431,7 +418,7 @@ hidden."
 	  (if (re-search-forward "[A-Za-z0-9]\\(\\.?[A-Za-z0-9]\\)*" nil t)
 	      (progn
 		(kproperty:set 'kcell
-			       (kotl-data:to-kcell-v3
+			       (kcell-data:to-kcell-v3
 				(aref kcell-vector kcell-num)))
 		(setq kcell-num (1+ kcell-num))))
 	  (search-forward "\n\n" nil t)))))
@@ -461,8 +448,7 @@ hidden."
   "Return a string containing OBJECT, any Lisp object, in pretty-printed form.
 Quoting characters are used when needed to make output that `read' can
 handle, whenever this is possible."
-  (save-excursion
-    (set-buffer (get-buffer-create " kfile:print-to-string"))
+  (with-current-buffer (get-buffer-create " kfile:print-to-string")
     (let ((emacs-lisp-mode-hook)
 	  (buffer-read-only))
       (erase-buffer)
@@ -480,14 +466,14 @@ handle, whenever this is possible."
 		  (forward-char 1)))
 	       ((and (looking-at "\\(quote[ \t]+\\)\\([^.)]\\)")
 		     (> (match-beginning 1) 1)
-		     (= ?\( (char-after (1- (match-beginning 1))))
+		     (eq ?\( (char-after (1- (match-beginning 1))))
 		     ;; Make sure this is a two-element list.
 		     (save-excursion
 		       (goto-char (match-beginning 2))
 		       (forward-sexp)
 		       ;; (looking-at "[ \t]*\)")
-		       ;; Avoid mucking with match-data; does this test work?
-		       (char-equal ?\) (char-after (point)))))
+		       ;; Avoid mucking with match-data
+		       (eq ?\) (char-after (point)))))
 		;; -1 gets the paren preceding the quote as well.
 		(delete-region (1- (match-beginning 1)) (match-end 1))
 		(insert "'")
@@ -504,7 +490,7 @@ handle, whenever this is possible."
 		(delete-region
 		 (point)
 		 (progn (skip-chars-forward " \t") (point)))
-		(if (not (char-equal ?' (char-after (1- (point)))))
+		(if (not (eq ?' (char-after (1- (point)))))
 		    (insert ?\n)))
 	       ((condition-case ()
 		    (prog1 t (up-list 1))
@@ -515,7 +501,7 @@ handle, whenever this is possible."
 		(delete-region
 		 (point)
 		 (progn (skip-chars-forward " \t") (point)))
-		(if (not (char-equal ?' (char-after (1- (point)))))
+		(if (not (eq ?' (char-after (1- (point)))))
 		    (insert ?\n)))
 	       (t (goto-char (point-max)))))
 	    (goto-char (point-min))
@@ -536,16 +522,9 @@ Output stream is STREAM, or value of `standard-output' (which see)."
     (while (not filename)
       (setq filename (read-file-name prompt nil nil existing-p))
       (if (or (null filename) (equal filename ""))
-	  (progn (ding) (setq filename nil))))
+	  (progn (beep) (setq filename nil))))
     filename))
 
-;;;
-;;; Private variables
-;;;
-
-(defvar kfile:escape-newlines t 
-  "Value of print-escape-newlines used by 'kfile:print-to-string' function.")
-
 (provide 'kfile)
 
 ;;; kfile.el ends here
